/*
 * XML.h
 *
 * Copyright (C) 2021 OPTOELECTRONICS CO.,LTD. All rights reserved.
 */

#ifndef _XML_H_
#define _XML_H_

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include "CodeConversion.h"

//////////////////////////////////////////////
// DEBUG log flags
//////////////////////////////////////////////
extern int DBG_XML_API;
extern int DBG_XML_PARSER;
extern int DBG_XML_DETAIL;

////////////////////////////////////////
// Structures
////////////////////////////////////////
typedef struct _XML_Attribute
{
	char *name;
	char *value;
	char *value_raw;
	unsigned int warning;
	struct _XML_Attribute *nextAttribute;
} XML_Attribute;

typedef struct _XML_node
{
	char *name;
	char *value;
	XML_Attribute *attribute;
	char *name_raw;
	char *value_raw;
	unsigned int warning;
	struct _XML_node *parentNode;
	struct _XML_node *childNode;
	struct _XML_node *nextNode;
	int signature;
} XML_Node;

//Value of XML_Attribute.warning and XML_Node.warning
#define XML_WARNING_UNSUPPORTED_CODE_IN_NAME		0x00000001
#define XML_WARNING_UNSUPPORTED_CODE_IN_VALUE		0x00000002

////////////////////////////////////////
// Error code
////////////////////////////////////////
#define XML_INVALID_PARAMETER_ERROR			CONV_INVALID_PARAMETER_ERROR		// 1
#define XML_MEMORY_ALLOCATION_ERROR			CONV_MEMORY_ALLOCATION_ERROR		// 2
											//CONV_INVALID_UTF8_FORMAT_ERROR	// 3
#define XML_FORMAT_ERROR					10
#define XML_FILE_ERROR					 	11
#define XML_INVALID_XML_OBJECT_ERROR		12
#define XML_INVALID_NODE_OBJECT_ERROR		13
#define XML_NOT_FOUND						14
#define XML_NOT_UNIQUE_INTERMEDIATE_NODE 	15
#define XML_NOT_UNIQUE_NODE				 	16
#define XML_UNICODE_CONVERSION_ERROR		17
#define XML_NO_VALUE_IN_INTERMEDIATE_NODE	18
#define XML_CHILD_NODE_DISALLOWED_ERROR 	19
#define XML_DELETE_NODE_ERROR			 	20
#define XML_UNSUPPORTED_CODE_WARNING		30


////////////////////////////////////////
// Functions
////////////////////////////////////////
typedef void * XML_HANDLE;
// Callback functions for XML_CreateXmlObject()
typedef char * (*XML_UTF8_TO_LOCAL_STRING)(char *src, int length, int *error_code, size_t *output_length);
typedef char * (*XML_LOCAL_TO_UTF8_STRING)(char *src, int length, int *error_code, size_t *output_length);

// src_type for XML_CreateXmlObject()
#define XML_LOCAL_STRING	1
#define XML_UTF_STRING		2
#define XML_UTF_FILE		3
XML_HANDLE XML_CreateXmlObject(int src_type, const char *src, int offset, int length, XML_LOCAL_TO_UTF8_STRING LocalToUtf8Func, XML_UTF8_TO_LOCAL_STRING Utf8ToLocalFunc, int *ErrCode);
bool XML_ReleaseXmlObject(XML_HANDLE hXml, int *ErrCode);

XML_Node *XML_GetRootNode(XML_HANDLE hXml, int *ErrCode);
char *XML_GetRootNodeName(XML_HANDLE hXml, int *ErrCode);

XML_Node *XML_FindNodeByPath(XML_HANDLE hXml, const char *nodePath, int *ErrCode);
XML_Node *XML_FindChildNodeByPath(XML_HANDLE hXml, XML_Node *parentNode, const char *childNodePath, int *ErrCode);
XML_Node *XML_NextNode(XML_Node *node, const char *name, int *ErrCode);
XML_Node *XML_ParentNode(XML_Node *node, int *ErrCode);

char *XML_GetNodeValue(XML_HANDLE hXml, XML_Node *node, int *ErrCode);
char *XML_GetNodeValueByPath(XML_HANDLE hXml, const char *nodePath, int *ErrCode);
char *XML_GetChildNodeValueByPath(XML_HANDLE hXml, XML_Node *parentNode, const char *childNodePath, int *ErrCode);

bool XML_SetNodeValue(XML_HANDLE hXml,  XML_Node *node, const char *value, int *ErrCode);
XML_Node *XML_SetNodeValueByPath(XML_HANDLE hXml, const char *nodePath, const char *value, int *ErrCode);
XML_Node *XML_SetChildNodeValueByPath(XML_HANDLE hXml, XML_Node *parentNode, const char *childNodePath, const char *value, int *ErrCode);

XML_Node *XML_AddChildNode(XML_HANDLE hXml, XML_Node *node, const char *name, const char *value, int *ErrCode);
bool XML_DeleteNode(XML_HANDLE hXml, XML_Node *node, bool delete_child_node, int *ErrCode);

XML_Attribute *XML_GetAttrubute(XML_HANDLE hXml, XML_Node *node, int *ErrCode);
XML_Attribute *XML_NextAttribute(XML_Attribute *attribute, int *ErrCode);
XML_Attribute *XML_FindAttrubute(XML_HANDLE hXml, XML_Node *node, const char *name, int *ErrCode);
XML_Attribute *XML_SetAttribute(XML_HANDLE hXml, XML_Node *node, const char *name, const char *value, int *ErrCode);
bool XML_DeleteAttributes(XML_HANDLE hXml, XML_Node *node, const char *name, int *ErrCode);

char *XML_SaveAsXmlMemory(XML_HANDLE hXml, const char *prolog, bool format, size_t *output_length, int *ErrCode);
bool XML_SaveAsXmlFile(XML_HANDLE hXml, const char *prolog, const char *filename, bool format, int *ErrCode);

void XML_DumpXmlObject(XML_HANDLE hXml);
void XML_DumpNode(XML_Node *node);

#endif /* _XML_H_ */
